@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright © 2016-2017 The TokTok team.
+ * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013 Tox project.
  *
  * This file is part of Tox, the free peer to peer instant messenger.
@@ -22,12 +22,14 @@
  * You should have received a copy of the GNU General Public License
  * along with Tox.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifndef MESSENGER_H
-#define MESSENGER_H
+#ifndef C_TOXCORE_TOXCORE_MESSENGER_H
+#define C_TOXCORE_TOXCORE_MESSENGER_H
 
 #include "friend_connection.h"
 #include "friend_requests.h"
 #include "logger.h"
+#include "net_crypto.h"
+#include "state.h"
 
 #define MAX_NAME_LENGTH 128
 /* TODO(irungentoo): this must depend on other variable. */
@@ -44,36 +46,30 @@
 
 #define FRIEND_ADDRESS_SIZE (CRYPTO_PUBLIC_KEY_SIZE + sizeof(uint32_t) + sizeof(uint16_t))
 
-enum {
+typedef enum Message_Type {
     MESSAGE_NORMAL,
     MESSAGE_ACTION
-};
+} Message_Type;
 
-/* NOTE: Packet ids below 24 must never be used. */
-#define PACKET_ID_ONLINE 24
-#define PACKET_ID_OFFLINE 25
-#define PACKET_ID_NICKNAME 48
-#define PACKET_ID_STATUSMESSAGE 49
-#define PACKET_ID_USERSTATUS 50
-#define PACKET_ID_TYPING 51
-#define PACKET_ID_MESSAGE 64
-#define PACKET_ID_ACTION (PACKET_ID_MESSAGE + MESSAGE_ACTION) /* 65 */
-#define PACKET_ID_MSI 69
-#define PACKET_ID_FILE_SENDREQUEST 80
-#define PACKET_ID_FILE_CONTROL 81
-#define PACKET_ID_FILE_DATA 82
-#define PACKET_ID_INVITE_CONFERENCE 96
-#define PACKET_ID_ONLINE_PACKET 97
-#define PACKET_ID_DIRECT_CONFERENCE 98
-#define PACKET_ID_MESSAGE_CONFERENCE 99
-#define PACKET_ID_LOSSY_CONFERENCE 199
+typedef struct Messenger Messenger;
 
-/* All packets starting with a byte in this range can be used for anything. */
-#define PACKET_ID_LOSSLESS_RANGE_START 160
-#define PACKET_ID_LOSSLESS_RANGE_SIZE 32
-#define PACKET_LOSSY_AV_RESERVED 8 /* Number of lossy packet types at start of range reserved for A/V. */
+// Returns the size of the data
+typedef uint32_t m_state_size_cb(const Messenger *m);
 
-typedef struct {
+// Returns the new pointer to data
+typedef uint8_t *m_state_save_cb(const Messenger *m, uint8_t *data);
+
+// Returns if there were any erros during loading
+typedef State_Load_Status m_state_load_cb(Messenger *m, const uint8_t *data, uint32_t length);
+
+typedef struct Messenger_State_Plugin {
+    State_Type type;
+    m_state_size_cb *size;
+    m_state_save_cb *save;
+    m_state_load_cb *load;
+} Messenger_State_Plugin;
+
+typedef struct Messenger_Options {
     bool ipv6enabled;
     bool udp_disabled;
     TCP_Proxy_Info proxy_info;
@@ -84,7 +80,11 @@ typedef struct {
     bool local_discovery_enabled;
 
     logger_cb *log_callback;
+    void *log_context;
     void *log_user_data;
+
+    Messenger_State_Plugin *state_plugins;
+    uint8_t state_plugins_length;
 } Messenger_Options;
 
 
@@ -95,18 +95,18 @@ struct Receipts {
 };
 
 /* Status definitions. */
-enum {
+typedef enum Friend_Status {
     NOFRIEND,
     FRIEND_ADDED,
     FRIEND_REQUESTED,
     FRIEND_CONFIRMED,
     FRIEND_ONLINE,
-};
+} Friend_Status;
 
 /* Errors for m_addfriend
  * FAERR - Friend Add Error
  */
-enum {
+typedef enum Friend_Add_Error {
     FAERR_TOOLONG = -1,
     FAERR_NOMESSAGE = -2,
     FAERR_OWNKEY = -3,
@@ -114,30 +114,29 @@ enum {
     FAERR_BADCHECKSUM = -6,
     FAERR_SETNEWNOSPAM = -7,
     FAERR_NOMEM = -8
-};
+} Friend_Add_Error;
 
 
 /* Default start timeout in seconds between friend requests. */
-#define FRIENDREQUEST_TIMEOUT 5;
+#define FRIENDREQUEST_TIMEOUT 5
 
-enum {
+typedef enum Connection_Status {
     CONNECTION_NONE,
     CONNECTION_TCP,
     CONNECTION_UDP,
     CONNECTION_UNKNOWN
-};
+} Connection_Status;
 
 /* USERSTATUS -
  * Represents userstatuses someone can have.
  */
 
-typedef enum {
+typedef enum Userstatus {
     USERSTATUS_NONE,
     USERSTATUS_AWAY,
     USERSTATUS_BUSY,
     USERSTATUS_INVALID
-}
-USERSTATUS;
+} Userstatus;
 
 #define FILE_ID_LENGTH 32
 
@@ -151,37 +150,74 @@ struct File_Transfers {
     unsigned int slots_allocated; /* number of slots allocated to this transfer. */
     uint8_t id[FILE_ID_LENGTH];
 };
-enum {
+typedef enum Filestatus {
     FILESTATUS_NONE,
     FILESTATUS_NOT_ACCEPTED,
     FILESTATUS_TRANSFERRING,
-    //FILESTATUS_BROKEN,
+    // FILESTATUS_BROKEN,
     FILESTATUS_FINISHED
-};
+} Filestatus;
 
-enum {
+typedef enum File_Pause {
     FILE_PAUSE_NOT,
     FILE_PAUSE_US,
     FILE_PAUSE_OTHER,
     FILE_PAUSE_BOTH
-};
+} File_Pause;
 
-enum {
+typedef enum Filecontrol {
     FILECONTROL_ACCEPT,
     FILECONTROL_PAUSE,
     FILECONTROL_KILL,
     FILECONTROL_SEEK
-};
+} Filecontrol;
 
-enum {
+typedef enum Filekind {
     FILEKIND_DATA,
     FILEKIND_AVATAR
-};
+} Filekind;
 
 
-typedef struct Messenger Messenger;
+typedef void m_self_connection_status_cb(Messenger *m, unsigned int connection_status, void *user_data);
+typedef void m_friend_status_cb(Messenger *m, uint32_t friend_number, unsigned int status, void *user_data);
+typedef void m_friend_connection_status_cb(Messenger *m, uint32_t friend_number, unsigned int connection_status,
+        void *user_data);
+typedef void m_friend_message_cb(Messenger *m, uint32_t friend_number, unsigned int message_type,
+                                 const uint8_t *message, size_t length, void *user_data);
+typedef void m_file_recv_control_cb(Messenger *m, uint32_t friend_number, uint32_t file_number, unsigned int control,
+                                    void *user_data);
+typedef void m_friend_request_cb(Messenger *m, const uint8_t *public_key, const uint8_t *message, size_t length,
+                                 void *user_data);
+typedef void m_friend_name_cb(Messenger *m, uint32_t friend_number, const uint8_t *name, size_t length,
+                              void *user_data);
+typedef void m_friend_status_message_cb(Messenger *m, uint32_t friend_number, const uint8_t *message, size_t length,
+                                        void *user_data);
+typedef void m_friend_typing_cb(Messenger *m, uint32_t friend_number, bool is_typing, void *user_data);
+typedef void m_friend_read_receipt_cb(Messenger *m, uint32_t friend_number, uint32_t message_id, void *user_data);
+typedef void m_file_recv_cb(Messenger *m, uint32_t friend_number, uint32_t file_number, uint32_t kind,
+                            uint64_t file_size, const uint8_t *filename, size_t filename_length, void *user_data);
+typedef void m_file_chunk_request_cb(Messenger *m, uint32_t friend_number, uint32_t file_number, uint64_t position,
+                                     size_t length, void *user_data);
+typedef void m_file_recv_chunk_cb(Messenger *m, uint32_t friend_number, uint32_t file_number, uint64_t position,
+                                  const uint8_t *data, size_t length, void *user_data);
+typedef void m_friend_lossy_packet_cb(Messenger *m, uint32_t friend_number, const uint8_t *data, size_t length,
+                                      void *user_data);
+typedef void m_friend_lossless_packet_cb(Messenger *m, uint32_t friend_number, const uint8_t *data, size_t length,
+        void *user_data);
+typedef void m_friend_connectionstatuschange_internal_cb(Messenger *m, uint32_t friend_number,
+        uint8_t connection_status, void *user_data);
+typedef void m_conference_invite_cb(Messenger *m, uint32_t friend_number, const uint8_t *cookie, uint16_t length,
+                                    void *user_data);
+typedef void m_msi_packet_cb(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length,
+                             void *user_data);
+typedef int m_lossy_rtp_packet_cb(Messenger *m, uint32_t friendnumber, const uint8_t *data, uint16_t len, void *object);
 
-typedef struct {
+typedef struct RTP_Packet_Handler {
+    m_lossy_rtp_packet_cb *function;
+    void *object;
+} RTP_Packet_Handler;
+
+typedef struct Friend {
     uint8_t real_pk[CRYPTO_PUBLIC_KEY_SIZE];
     int friendcon_id;
 
@@ -195,7 +231,7 @@ typedef struct {
     uint8_t statusmessage[MAX_STATUSMESSAGE_LENGTH];
     uint16_t statusmessage_length;
     uint8_t statusmessage_sent;
-    USERSTATUS userstatus;
+    Userstatus userstatus;
     uint8_t userstatus_sent;
     uint8_t user_istyping;
     uint8_t user_istyping_sent;
@@ -209,10 +245,7 @@ typedef struct {
     uint32_t num_sending_files;
     struct File_Transfers file_receiving[MAX_CONCURRENT_FILE_PIPES];
 
-    struct {
-        int (*function)(Messenger *m, uint32_t friendnumber, const uint8_t *data, uint16_t len, void *object);
-        void *object;
-    } lossy_rtp_packethandlers[PACKET_LOSSY_AV_RESERVED];
+    RTP_Packet_Handler lossy_rtp_packethandlers[PACKET_ID_RANGE_LOSSY_AV_SIZE];
 
     struct Receipts *receipts_start;
     struct Receipts *receipts_end;
@@ -220,6 +253,7 @@ typedef struct {
 
 struct Messenger {
     Logger *log;
+    Mono_Time *mono_time;
 
     Networking_Core *net;
     Net_Crypto *net_crypto;
@@ -239,42 +273,43 @@ struct Messenger {
     uint8_t statusmessage[MAX_STATUSMESSAGE_LENGTH];
     uint16_t statusmessage_length;
 
-    USERSTATUS userstatus;
+    Userstatus userstatus;
 
     Friend *friendlist;
     uint32_t numfriends;
 
     time_t lastdump;
 
-    uint8_t has_added_relays; // If the first connection has occurred in do_messenger
+    bool has_added_relays; // If the first connection has occurred in do_messenger
+
+    uint16_t num_loaded_relays;
     Node_format loaded_relays[NUM_SAVED_TCP_RELAYS]; // Relays loaded from config
 
-    void (*friend_message)(struct Messenger *m, uint32_t, unsigned int, const uint8_t *, size_t, void *);
-    void (*friend_namechange)(struct Messenger *m, uint32_t, const uint8_t *, size_t, void *);
-    void (*friend_statusmessagechange)(struct Messenger *m, uint32_t, const uint8_t *, size_t, void *);
-    void (*friend_userstatuschange)(struct Messenger *m, uint32_t, unsigned int, void *);
-    void (*friend_typingchange)(struct Messenger *m, uint32_t, bool, void *);
-    void (*read_receipt)(struct Messenger *m, uint32_t, uint32_t, void *);
-    void (*friend_connectionstatuschange)(struct Messenger *m, uint32_t, unsigned int, void *);
-    void (*friend_connectionstatuschange_internal)(struct Messenger *m, uint32_t, uint8_t, void *);
+    m_friend_message_cb *friend_message;
+    m_friend_name_cb *friend_namechange;
+    m_friend_status_message_cb *friend_statusmessagechange;
+    m_friend_status_cb *friend_userstatuschange;
+    m_friend_typing_cb *friend_typingchange;
+    m_friend_read_receipt_cb *read_receipt;
+    m_friend_connection_status_cb *friend_connectionstatuschange;
+    m_friend_connectionstatuschange_internal_cb *friend_connectionstatuschange_internal;
     void *friend_connectionstatuschange_internal_userdata;
 
-    void *conferences_object; /* Set by new_groupchats()*/
-    void (*conference_invite)(struct Messenger *m, uint32_t, const uint8_t *, uint16_t, void *);
+    struct Group_Chats *conferences_object; /* Set by new_groupchats()*/
+    m_conference_invite_cb *conference_invite;
 
-    void (*file_sendrequest)(struct Messenger *m, uint32_t, uint32_t, uint32_t, uint64_t, const uint8_t *, size_t,
-                             void *);
-    void (*file_filecontrol)(struct Messenger *m, uint32_t, uint32_t, unsigned int, void *);
-    void (*file_filedata)(struct Messenger *m, uint32_t, uint32_t, uint64_t, const uint8_t *, size_t, void *);
-    void (*file_reqchunk)(struct Messenger *m, uint32_t, uint32_t, uint64_t, size_t, void *);
+    m_file_recv_cb *file_sendrequest;
+    m_file_recv_control_cb *file_filecontrol;
+    m_file_recv_chunk_cb *file_filedata;
+    m_file_chunk_request_cb *file_reqchunk;
 
-    void (*msi_packet)(struct Messenger *m, uint32_t, const uint8_t *, uint16_t, void *);
+    m_msi_packet_cb *msi_packet;
     void *msi_packet_userdata;
 
-    void (*lossy_packethandler)(struct Messenger *m, uint32_t, const uint8_t *, size_t, void *);
-    void (*lossless_packethandler)(struct Messenger *m, uint32_t, const uint8_t *, size_t, void *);
+    m_friend_lossy_packet_cb *lossy_packethandler;
+    m_friend_lossless_packet_cb *lossless_packethandler;
 
-    void (*core_connection_change)(struct Messenger *m, unsigned int, void *);
+    m_self_connection_status_cb *core_connection_change;
     unsigned int last_connection_status;
 
     Messenger_Options options;
@@ -440,9 +475,9 @@ int m_get_self_statusmessage_size(const Messenger *m);
 int m_copy_statusmessage(const Messenger *m, int32_t friendnumber, uint8_t *buf, uint32_t maxlen);
 int m_copy_self_statusmessage(const Messenger *m, uint8_t *buf);
 
-/*  return one of USERSTATUS values.
+/*  return one of Userstatus values.
  *  Values unknown to your application should be represented as USERSTATUS_NONE.
- *  As above, the self variant will return our own USERSTATUS.
+ *  As above, the self variant will return our own Userstatus.
  *  If friendnumber is invalid, this shall return USERSTATUS_INVALID.
  */
 uint8_t m_get_userstatus(const Messenger *m, int32_t friendnumber);
@@ -469,44 +504,38 @@ int m_set_usertyping(Messenger *m, int32_t friendnumber, uint8_t is_typing);
  */
 int m_get_istyping(const Messenger *m, int32_t friendnumber);
 
-/* Set the logger callback.
- */
-void m_callback_log(Messenger *m, logger_cb *function, void *context, void *userdata);
-
 /* Set the function that will be executed when a friend request is received.
  *  Function format is function(uint8_t * public_key, uint8_t * data, size_t length)
  */
-void m_callback_friendrequest(Messenger *m, void (*function)(Messenger *m, const uint8_t *, const uint8_t *, size_t,
-                              void *));
+void m_callback_friendrequest(Messenger *m, m_friend_request_cb *function);
 
 /* Set the function that will be executed when a message from a friend is received.
  *  Function format is: function(uint32_t friendnumber, unsigned int type, uint8_t * message, uint32_t length)
  */
-void m_callback_friendmessage(Messenger *m, void (*function)(Messenger *m, uint32_t, unsigned int, const uint8_t *,
-                              size_t, void *));
+void m_callback_friendmessage(Messenger *m, m_friend_message_cb *function);
 
 /* Set the callback for name changes.
  *  Function(uint32_t friendnumber, uint8_t *newname, size_t length)
  *  You are not responsible for freeing newname.
  */
-void m_callback_namechange(Messenger *m, void (*function)(Messenger *m, uint32_t, const uint8_t *, size_t, void *));
+void m_callback_namechange(Messenger *m, m_friend_name_cb *function);
 
 /* Set the callback for status message changes.
  *  Function(uint32_t friendnumber, uint8_t *newstatus, size_t length)
  *
  *  You are not responsible for freeing newstatus
  */
-void m_callback_statusmessage(Messenger *m, void (*function)(Messenger *m, uint32_t, const uint8_t *, size_t, void *));
+void m_callback_statusmessage(Messenger *m, m_friend_status_message_cb *function);
 
 /* Set the callback for status type changes.
- *  Function(uint32_t friendnumber, USERSTATUS kind)
+ *  Function(uint32_t friendnumber, Userstatus kind)
  */
-void m_callback_userstatus(Messenger *m, void (*function)(Messenger *m, uint32_t, unsigned int, void *));
+void m_callback_userstatus(Messenger *m, m_friend_status_cb *function);
 
 /* Set the callback for typing changes.
  *  Function(uint32_t friendnumber, uint8_t is_typing)
  */
-void m_callback_typingchange(Messenger *m, void(*function)(Messenger *m, uint32_t, bool, void *));
+void m_callback_typingchange(Messenger *m, m_friend_typing_cb *function);
 
 /* Set the callback for read receipts.
  *  Function(uint32_t friendnumber, uint32_t receipt)
@@ -517,7 +546,7 @@ void m_callback_typingchange(Messenger *m, void(*function)(Messenger *m, uint32_
  *  Since core doesn't track ids for you, receipt may not correspond to any message.
  *  In that case, you should discard it.
  */
-void m_callback_read_receipt(Messenger *m, void (*function)(Messenger *m, uint32_t, uint32_t, void *));
+void m_callback_read_receipt(Messenger *m, m_friend_read_receipt_cb *function);
 
 /* Set the callback for connection status changes.
  *  function(uint32_t friendnumber, uint8_t status)
@@ -530,17 +559,17 @@ void m_callback_read_receipt(Messenger *m, void (*function)(Messenger *m, uint32
  *  being previously online" part.
  *  It's assumed that when adding friends, their connection status is offline.
  */
-void m_callback_connectionstatus(Messenger *m, void (*function)(Messenger *m, uint32_t, unsigned int, void *));
+void m_callback_connectionstatus(Messenger *m, m_friend_connection_status_cb *function);
 
 /* Same as previous but for internal A/V core usage only */
-void m_callback_connectionstatus_internal_av(Messenger *m, void (*function)(Messenger *m, uint32_t, uint8_t, void *),
+void m_callback_connectionstatus_internal_av(Messenger *m, m_friend_connectionstatuschange_internal_cb *function,
         void *userdata);
 
 
 /* Set the callback for typing changes.
  *  Function(unsigned int connection_status (0 = not connected, 1 = TCP only, 2 = UDP + TCP))
  */
-void m_callback_core_connection(Messenger *m, void (*function)(Messenger *m, unsigned int, void *));
+void m_callback_core_connection(Messenger *m, m_self_connection_status_cb *function);
 
 /**********CONFERENCES************/
 
@@ -548,8 +577,7 @@ void m_callback_core_connection(Messenger *m, void (*function)(Messenger *m, uns
  *
  *  Function(Messenger *m, uint32_t friendnumber, uint8_t *data, uint16_t length, void *userdata)
  */
-void m_callback_conference_invite(Messenger *m, void (*function)(Messenger *m, uint32_t, const uint8_t *, uint16_t,
-                                  void *));
+void m_callback_conference_invite(Messenger *m, m_conference_invite_cb *function);
 
 /* Send a conference invite packet.
  *
@@ -565,8 +593,7 @@ int send_conference_invite_packet(const Messenger *m, int32_t friendnumber, cons
  *
  *  Function(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint32_t filetype, uint64_t filesize, uint8_t *filename, size_t filename_length, void *userdata)
  */
-void callback_file_sendrequest(Messenger *m, void (*function)(Messenger *m,  uint32_t, uint32_t, uint32_t, uint64_t,
-                               const uint8_t *, size_t, void *));
+void callback_file_sendrequest(Messenger *m, m_file_recv_cb *function);
 
 
 /* Set the callback for file control requests.
@@ -574,22 +601,21 @@ void callback_file_sendrequest(Messenger *m, void (*function)(Messenger *m,  uin
  *  Function(Tox *tox, uint32_t friendnumber, uint32_t filenumber, unsigned int control_type, void *userdata)
  *
  */
-void callback_file_control(Messenger *m, void (*function)(Messenger *m, uint32_t, uint32_t, unsigned int, void *));
+void callback_file_control(Messenger *m, m_file_recv_control_cb *function);
 
 /* Set the callback for file data.
  *
  *  Function(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint64_t position, uint8_t *data, size_t length, void *userdata)
  *
  */
-void callback_file_data(Messenger *m, void (*function)(Messenger *m, uint32_t, uint32_t, uint64_t, const uint8_t *,
-                        size_t, void *));
+void callback_file_data(Messenger *m, m_file_recv_chunk_cb *function);
 
 /* Set the callback for file request chunk.
  *
  *  Function(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint64_t position, size_t length, void *userdata)
  *
  */
-void callback_file_reqchunk(Messenger *m, void (*function)(Messenger *m, uint32_t, uint32_t, uint64_t, size_t, void *));
+void callback_file_reqchunk(Messenger *m, m_file_chunk_request_cb *function);
 
 
 /* Copy the file transfer file id to file_id
@@ -668,8 +694,7 @@ uint64_t file_dataremaining(const Messenger *m, int32_t friendnumber, uint8_t fi
  *
  *  Function(Messenger *m, uint32_t friendnumber, uint8_t *data, uint16_t length, void *userdata)
  */
-void m_callback_msi_packet(Messenger *m, void (*function)(Messenger *m, uint32_t, const uint8_t *, uint16_t, void *),
-                           void *userdata);
+void m_callback_msi_packet(Messenger *m, m_msi_packet_cb *function, void *userdata);
 
 /* Send an msi packet.
  *
@@ -683,16 +708,15 @@ int m_msi_packet(const Messenger *m, int32_t friendnumber, const uint8_t *data,
  * return -1 on failure.
  * return 0 on success.
  */
-int m_callback_rtp_packet(Messenger *m, int32_t friendnumber, uint8_t byte, int (*packet_handler_callback)(Messenger *m,
-                          uint32_t friendnumber, const uint8_t *data, uint16_t len, void *object), void *object);
+int m_callback_rtp_packet(Messenger *m, int32_t friendnumber, uint8_t byte,
+                          m_lossy_rtp_packet_cb *function, void *object);
 
 /**********************************************/
 
 /* Set handlers for custom lossy packets.
  *
  */
-void custom_lossy_packet_registerhandler(Messenger *m, void (*packet_handler_callback)(Messenger *m,
-        uint32_t friendnumber, const uint8_t *data, size_t len, void *object));
+void custom_lossy_packet_registerhandler(Messenger *m, m_friend_lossy_packet_cb *lossy_packethandler);
 
 /* High level function to send custom lossy packets.
  *
@@ -709,8 +733,7 @@ int m_send_custom_lossy_packet(const Messenger *m, int32_t friendnumber, const u
 /* Set handlers for custom lossless packets.
  *
  */
-void custom_lossless_packet_registerhandler(Messenger *m, void (*packet_handler_callback)(Messenger *m,
-        uint32_t friendnumber, const uint8_t *data, size_t len, void *object));
+void custom_lossless_packet_registerhandler(Messenger *m, m_friend_lossless_packet_cb *lossless_packethandler);
 
 /* High level function to send custom lossless packets.
  *
@@ -725,12 +748,12 @@ int send_custom_lossless_packet(const Messenger *m, int32_t friendnumber, const
 
 /**********************************************/
 
-enum {
+typedef enum Messenger_Error {
     MESSENGER_ERROR_NONE,
     MESSENGER_ERROR_PORT,
     MESSENGER_ERROR_TCP_SERVER,
     MESSENGER_ERROR_OTHER
-};
+} Messenger_Error;
 
 /* Run this at startup.
  *  return allocated instance of Messenger on success.
@@ -738,7 +761,7 @@ enum {
  *
  *  if error is not NULL it will be set to one of the values in the enum above.
  */
-Messenger *new_messenger(Messenger_Options *options, unsigned int *error);
+Messenger *new_messenger(Mono_Time *mono_time, Messenger_Options *options, unsigned int *error);
 
 /* Run this before closing shop
  * Free all datastructures.
@@ -757,14 +780,30 @@ uint32_t messenger_run_interval(const Messenger *m);
 
 /* SAVING AND LOADING FUNCTIONS: */
 
+/* Registers a state plugin for saving, loadding, and getting the size of a section of the save
+ *
+ * returns true on success
+ * returns false on error
+ */
+bool m_register_state_plugin(Messenger *m, State_Type type, m_state_size_cb size_callback,
+                             m_state_load_cb load_callback, m_state_save_cb save_callback);
+
 /* return size of the messenger data (for saving). */
 uint32_t messenger_size(const Messenger *m);
 
-/* Save the messenger in data (must be allocated memory of size Messenger_size()) */
-void messenger_save(const Messenger *m, uint8_t *data);
+/* Save the messenger in data (must be allocated memory of size at least Messenger_size()) */
+uint8_t *messenger_save(const Messenger *m, uint8_t *data);
 
-/* Load the messenger from data of size length. */
-int messenger_load(Messenger *m, const uint8_t *data, uint32_t length);
+/* Load a state section.
+ *
+ * @param data Data to load.
+ * @param length Length of data.
+ * @param type Type of section (STATE_TYPE_*).
+ * @param status Result of loading section is stored here if the section is handled.
+ * @return true iff section handled.
+ */
+bool messenger_load_state_section(Messenger *m, const uint8_t *data, uint32_t length, uint16_t type,
+                                  State_Load_Status *status);
 
 /* Return the number of friends in the instance m.
  * You should use this to determine how much memory to allocate
