@@ -118,6 +118,8 @@ static bool realloc_conferences(Group_Chats *g_c, uint16_t num)
 static void setup_conference(Group_c *g)
 {
     memset(g, 0, sizeof(Group_c));
+
+    g->maxfrozen = MAX_FROZEN_DEFAULT;
 }
 
 /* Create a new empty groupchat connection.
@@ -539,9 +541,7 @@ static int note_peer_active(Group_Chats *g_c, uint32_t groupnumber, uint16_t pee
 
     ++g->numpeers;
 
-    if (!delete_frozen(g, frozen_index)) {
-        return -1;
-    }
+    delete_frozen(g, frozen_index);
 
     if (g_c->peer_list_changed_callback) {
         g_c->peer_list_changed_callback(g_c->m, groupnumber, userdata);
@@ -646,8 +646,8 @@ static int addpeer(Group_Chats *g_c, uint32_t groupnumber, const uint8_t *real_p
     id_copy(g->group[g->numpeers].temp_pk, temp_pk);
     g->group[g->numpeers].temp_pk_updated = true;
     g->group[g->numpeers].peer_number = peer_number;
-
     g->group[g->numpeers].last_active = mono_time_get(g_c->mono_time);
+    g->group[g->numpeers].is_friend = (getfriend_id(g_c->m, real_pk) != -1);
     ++g->numpeers;
 
     add_to_closest(g_c, groupnumber, real_pk, temp_pk);
@@ -754,6 +754,56 @@ static int delpeer(Group_Chats *g_c, uint32_t groupnumber, int peer_index, void
     return 0;
 }
 
+static int cmp_u64(uint64_t a, uint64_t b)
+{
+    return (a > b) - (a < b);
+}
+
+/* Order peers with friends first and with more recently active earlier */
+static int cmp_frozen(const void *a, const void *b)
+{
+    const Group_Peer *pa = (const Group_Peer *) a;
+    const Group_Peer *pb = (const Group_Peer *) b;
+
+    if (pa->is_friend ^ pb->is_friend) {
+        return pa->is_friend ? -1 : 1;
+    }
+
+    return cmp_u64(pb->last_active, pa->last_active);
+}
+
+/* Delete frozen peers as necessary to ensure at most g->maxfrozen remain.
+ *
+ * return true if any frozen peers are removed.
+ */
+static bool delete_old_frozen(Group_c *g)
+{
+    if (g->numfrozen <= g->maxfrozen) {
+        return false;
+    }
+
+    if (g->maxfrozen == 0) {
+        free(g->frozen);
+        g->frozen = nullptr;
+        g->numfrozen = 0;
+        return true;
+    }
+
+    qsort(g->frozen, g->numfrozen, sizeof(Group_Peer), cmp_frozen);
+
+    Group_Peer *temp = (Group_Peer *)realloc(g->frozen, sizeof(Group_Peer) * g->maxfrozen);
+
+    if (temp == nullptr) {
+        return false;
+    }
+
+    g->frozen = temp;
+
+    g->numfrozen = g->maxfrozen;
+
+    return true;
+}
+
 static bool try_send_rejoin(Group_Chats *g_c, uint32_t groupnumber, const uint8_t *real_pk);
 
 static int freeze_peer(Group_Chats *g_c, uint32_t groupnumber, int peer_index, void *userdata)
@@ -764,8 +814,6 @@ static int freeze_peer(Group_Chats *g_c, uint32_t groupnumber, int peer_index, v
         return -1;
     }
 
-    try_send_rejoin(g_c, groupnumber, g->group[peer_index].real_pk);
-
     Group_Peer *temp = (Group_Peer *)realloc(g->frozen, sizeof(Group_Peer) * (g->numfrozen + 1));
 
     if (temp == nullptr) {
@@ -774,9 +822,19 @@ static int freeze_peer(Group_Chats *g_c, uint32_t groupnumber, int peer_index, v
 
     g->frozen = temp;
     g->frozen[g->numfrozen] = g->group[peer_index];
+    g->frozen[g->numfrozen].object = nullptr;
+
+    if (delpeer(g_c, groupnumber, peer_index, userdata, true) != 0) {
+        return -1;
+    }
+
+    try_send_rejoin(g_c, groupnumber, g->frozen[g->numfrozen].real_pk);
+
     ++g->numfrozen;
 
-    return delpeer(g_c, groupnumber, peer_index, userdata, true);
+    delete_old_frozen(g);
+
+    return 0;
 }
 
 
@@ -1237,6 +1295,24 @@ int group_frozen_last_active(const Group_Chats *g_c, uint32_t groupnumber, int p
     return 0;
 }
 
+/* Set maximum number of frozen peers.
+ *
+ * return 0 on success.
+ * return -1 if groupnumber is invalid.
+ */
+int group_set_max_frozen(const Group_Chats *g_c, uint32_t groupnumber, uint32_t maxfrozen)
+{
+    Group_c *g = get_group_c(g_c, groupnumber);
+
+    if (!g) {
+        return -1;
+    }
+
+    g->maxfrozen = maxfrozen;
+    delete_old_frozen(g);
+    return 0;
+}
+
 /* List all the (frozen, if frozen is true) peers in the group chat.
  *
  * Copies the names of the peers to the name[length][MAX_NAME_LENGTH] array.
@@ -2831,6 +2907,12 @@ static unsigned int lossy_packet_not_received(const Group_c *g, int peer_index,
 
 }
 
+/* Does this group type make use of lossy packets? */
+static bool type_uses_lossy(uint8_t type)
+{
+    return (type == GROUPCHAT_TYPE_AV);
+}
+
 static int handle_lossy(void *object, int friendcon_id, const uint8_t *data, uint16_t length, void *userdata)
 {
     Group_Chats *g_c = (Group_Chats *)object;
@@ -2857,6 +2939,10 @@ static int handle_lossy(void *object, int friendcon_id, const uint8_t *data, uin
         return -1;
     }
 
+    if (!type_uses_lossy(g->type)) {
+        return -1;
+    }
+
     const int index = friend_in_close(g, friendcon_id);
 
     if (index == -1) {
@@ -2883,6 +2969,8 @@ static int handle_lossy(void *object, int friendcon_id, const uint8_t *data, uin
     ++lossy_data;
     --lossy_length;
 
+    send_lossy_all_close(g_c, groupnumber, data + 1 + sizeof(uint16_t), length - (1 + sizeof(uint16_t)), index);
+
     if (g_c->lossy_packethandlers[message_id].function) {
         if (g_c->lossy_packethandlers[message_id].function(g->object, groupnumber, peer_index, g->group[peer_index].object,
                 lossy_data, lossy_length) == -1) {
@@ -2892,7 +2980,6 @@ static int handle_lossy(void *object, int friendcon_id, const uint8_t *data, uin
         return -1;
     }
 
-    send_lossy_all_close(g_c, groupnumber, data + 1 + sizeof(uint16_t), length - (1 + sizeof(uint16_t)), index);
     return 0;
 }
 
@@ -2934,7 +3021,7 @@ int group_peer_set_object(const Group_Chats *g_c, uint32_t groupnumber, int peer
     return 0;
 }
 
-/* Return the object tide to the group chat previously set by group_set_object.
+/* Return the object tied to the group chat previously set by group_set_object.
  *
  * return NULL on failure.
  * return object on success.
@@ -2950,7 +3037,7 @@ void *group_get_object(const Group_Chats *g_c, uint32_t groupnumber)
     return g->object;
 }
 
-/* Return the object tide to the group chat peer previously set by group_peer_set_object.
+/* Return the object tied to the group chat peer previously set by group_peer_set_object.
  *
  * return NULL on failure.
  * return object on success.
@@ -3198,10 +3285,12 @@ static State_Load_Status load_conferences(Group_Chats *g_c, const uint8_t *data,
         lendian_bytes_to_host32(&g->numfrozen, data);
         data += sizeof(uint32_t);
 
-        g->frozen = (Group_Peer *)malloc(sizeof(Group_Peer) * g->numfrozen);
+        if (g->numfrozen > 0) {
+            g->frozen = (Group_Peer *)malloc(sizeof(Group_Peer) * g->numfrozen);
 
-        if (g->frozen == nullptr) {
-            return STATE_LOAD_STATUS_ERROR;
+            if (g->frozen == nullptr) {
+                return STATE_LOAD_STATUS_ERROR;
+            }
         }
 
         g->title_len = *data;
@@ -3242,6 +3331,13 @@ static State_Load_Status load_conferences(Group_Chats *g_c, const uint8_t *data,
 
             memcpy(peer->nick, data, peer->nick_len);
             data += peer->nick_len;
+
+            // NOTE: this relies on friends being loaded before conferences.
+            peer->is_friend = (getfriend_id(g_c->m, peer->real_pk) != -1);
+        }
+
+        if (g->numfrozen > g->maxfrozen) {
+            g->maxfrozen = g->numfrozen;
         }
 
         g->status = GROUPCHAT_STATUS_CONNECTED;
