@@ -1,3 +1,6 @@
+#ifndef C_TOXCORE_TOXENCRYPTSAVE_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_EXPORT_H
+#define C_TOXCORE_TOXENCRYPTSAVE_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_EXPORT_H
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -36,3 +39,5 @@
 #endif
 
 #endif
+
+#endif
